@@ -104,12 +104,6 @@ typedef struct _GdkAtom            *GdkAtom;
  */
 #define GDK_POINTER_TO_ATOM(ptr)  ((GdkAtom)(ptr))
 
-#ifdef GDK_NATIVE_WINDOW_POINTER
-#define GDK_GPOINTER_TO_NATIVE_WINDOW(p) ((GdkNativeWindow) (p))
-#else
-#define GDK_GPOINTER_TO_NATIVE_WINDOW(p) GPOINTER_TO_UINT(p)
-#endif
-
 #define _GDK_MAKE_ATOM(val) ((GdkAtom)GUINT_TO_POINTER(val))
 
 /**
@@ -120,18 +114,6 @@ typedef struct _GdkAtom            *GdkAtom;
  */
 #define GDK_NONE            _GDK_MAKE_ATOM (0)
 
-/**
- * GdkNativeWindow:
- *
- * Used to represent native windows (<type>Window</type>s for the X11
- * backend, <type>HWND</type>s for Win32).
- */
-#ifdef GDK_NATIVE_WINDOW_POINTER
-typedef gpointer GdkNativeWindow;
-#else
-typedef guint32 GdkNativeWindow;
-#endif
-
 /* Forward declarations of commonly used types */
 typedef struct _GdkColor              GdkColor;
 typedef struct _GdkRGBA               GdkRGBA;
