@@ -26,6 +26,7 @@
 #include "gdkasync.h"
 #include "gdkprivate-x11.h"
 
+#include <stdlib.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/extensions/XInput2.h>
@@ -362,6 +363,8 @@ gdk_x11_device_xi2_query_state (GdkDevice        *device,
   if (mask)
     *mask = _gdk_x11_device_xi2_translate_state (&mod_state, &button_state);
 
+  free (button_state.mask);
+
   return TRUE;
 }
 
@@ -449,7 +452,7 @@ gdk_x11_device_xi2_window_at_position (GdkDevice       *device,
   GdkWindow *window;
   Window xwindow, root, child, last = None;
   gdouble xroot_x, xroot_y, xwin_x, xwin_y;
-  XIButtonState button_state;
+  XIButtonState button_state = { 0 };
   XIModifierState mod_state;
   XIGroupState group_state;
 
@@ -501,6 +504,10 @@ gdk_x11_device_xi2_window_at_position (GdkDevice       *device,
             {
               window = GDK_WINDOW (list->data);
               xwindow = GDK_WINDOW_XID (window);
+
+              /* Free previous button mask, if any */
+              g_free (button_state.mask);
+
               gdk_x11_display_error_trap_push (display);
               XIQueryPointer (xdisplay,
                               device_xi2->device_id,
@@ -525,6 +532,8 @@ gdk_x11_device_xi2_window_at_position (GdkDevice       *device,
                   XSetWindowAttributes attributes;
                   Window w;
 
+                  free (button_state.mask);
+
                   w = XCreateWindow (xdisplay, xwindow, (int)xwin_x, (int)xwin_y, 1, 1, 0,
                                      CopyFromParent, InputOnly, CopyFromParent,
                                      0, &attributes);
@@ -558,6 +567,8 @@ gdk_x11_device_xi2_window_at_position (GdkDevice       *device,
   while (xwindow)
     {
       last = xwindow;
+      free (button_state.mask);
+
       gdk_x11_display_error_trap_push (display);
       XIQueryPointer (xdisplay,
                       device_xi2->device_id,
@@ -593,6 +604,8 @@ gdk_x11_device_xi2_window_at_position (GdkDevice       *device,
   if (mask)
     *mask = _gdk_x11_device_xi2_translate_state (&mod_state, &button_state);
 
+  free (button_state.mask);
+
   return window;
 }
 
@@ -677,7 +690,7 @@ _gdk_x11_device_xi2_translate_state (XIModifierState *mods_state,
   guint state = 0;
 
   if (mods_state)
-    state = (guint) mods_state->effective;
+    state = (guint) mods_state->base | mods_state->latched | mods_state->locked;
 
   if (buttons_state)
     {
