@@ -24,7 +24,7 @@
 #include "gtkicontheme.h"
 #include "gtkintl.h"
 #include "gtkprivate.h"
-#include "gtkalias.h"
+
 
 static void gtk_cell_renderer_pixbuf_get_property  (GObject                    *object,
 						    guint                       param_id,
@@ -39,17 +39,16 @@ static void gtk_cell_renderer_pixbuf_create_stock_pixbuf (GtkCellRendererPixbuf
 							  GtkWidget             *widget);
 static void gtk_cell_renderer_pixbuf_get_size   (GtkCellRenderer            *cell,
 						 GtkWidget                  *widget,
-						 GdkRectangle               *rectangle,
+						 const GdkRectangle         *rectangle,
 						 gint                       *x_offset,
 						 gint                       *y_offset,
 						 gint                       *width,
 						 gint                       *height);
 static void gtk_cell_renderer_pixbuf_render     (GtkCellRenderer            *cell,
-						 GdkDrawable                *window,
+						 cairo_t                    *cr,
 						 GtkWidget                  *widget,
-						 GdkRectangle               *background_area,
-						 GdkRectangle               *cell_area,
-						 GdkRectangle               *expose_area,
+						 const GdkRectangle         *background_area,
+						 const GdkRectangle         *cell_area,
 						 GtkCellRendererState        flags);
 
 
@@ -67,27 +66,37 @@ enum {
 };
 
 
-#define GTK_CELL_RENDERER_PIXBUF_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GTK_TYPE_CELL_RENDERER_PIXBUF, GtkCellRendererPixbufPrivate))
-
-typedef struct _GtkCellRendererPixbufPrivate GtkCellRendererPixbufPrivate;
 struct _GtkCellRendererPixbufPrivate
 {
-  gchar *stock_id;
   GtkIconSize stock_size;
-  gchar *stock_detail;
-  gboolean follow_state;
-  gchar *icon_name;
+
+  GdkPixbuf *pixbuf;
+  GdkPixbuf *pixbuf_expander_open;
+  GdkPixbuf *pixbuf_expander_closed;
+
   GIcon *gicon;
+
+  gboolean follow_state;
+
+  gchar *stock_id;
+  gchar *stock_detail;
+  gchar *icon_name;
 };
 
+
 G_DEFINE_TYPE (GtkCellRendererPixbuf, gtk_cell_renderer_pixbuf, GTK_TYPE_CELL_RENDERER)
 
+
 static void
 gtk_cell_renderer_pixbuf_init (GtkCellRendererPixbuf *cellpixbuf)
 {
   GtkCellRendererPixbufPrivate *priv;
 
-  priv = GTK_CELL_RENDERER_PIXBUF_GET_PRIVATE (cellpixbuf);
+  cellpixbuf->priv = G_TYPE_INSTANCE_GET_PRIVATE (cellpixbuf,
+                                                  GTK_TYPE_CELL_RENDERER_PIXBUF,
+                                                  GtkCellRendererPixbufPrivate);
+  priv = cellpixbuf->priv;
+
   priv->stock_size = GTK_ICON_SIZE_MENU;
 }
 
@@ -216,16 +225,14 @@ static void
 gtk_cell_renderer_pixbuf_finalize (GObject *object)
 {
   GtkCellRendererPixbuf *cellpixbuf = GTK_CELL_RENDERER_PIXBUF (object);
-  GtkCellRendererPixbufPrivate *priv;
+  GtkCellRendererPixbufPrivate *priv = cellpixbuf->priv;
 
-  priv = GTK_CELL_RENDERER_PIXBUF_GET_PRIVATE (object);
-  
-  if (cellpixbuf->pixbuf)
-    g_object_unref (cellpixbuf->pixbuf);
-  if (cellpixbuf->pixbuf_expander_open)
-    g_object_unref (cellpixbuf->pixbuf_expander_open);
-  if (cellpixbuf->pixbuf_expander_closed)
-    g_object_unref (cellpixbuf->pixbuf_expander_closed);
+  if (priv->pixbuf)
+    g_object_unref (priv->pixbuf);
+  if (priv->pixbuf_expander_open)
+    g_object_unref (priv->pixbuf_expander_open);
+  if (priv->pixbuf_expander_closed)
+    g_object_unref (priv->pixbuf_expander_closed);
 
   g_free (priv->stock_id);
   g_free (priv->stock_detail);
@@ -244,20 +251,18 @@ gtk_cell_renderer_pixbuf_get_property (GObject        *object,
 				       GParamSpec     *pspec)
 {
   GtkCellRendererPixbuf *cellpixbuf = GTK_CELL_RENDERER_PIXBUF (object);
-  GtkCellRendererPixbufPrivate *priv;
+  GtkCellRendererPixbufPrivate *priv = cellpixbuf->priv;
 
-  priv = GTK_CELL_RENDERER_PIXBUF_GET_PRIVATE (object);
-  
   switch (param_id)
     {
     case PROP_PIXBUF:
-      g_value_set_object (value, cellpixbuf->pixbuf);
+      g_value_set_object (value, priv->pixbuf);
       break;
     case PROP_PIXBUF_EXPANDER_OPEN:
-      g_value_set_object (value, cellpixbuf->pixbuf_expander_open);
+      g_value_set_object (value, priv->pixbuf_expander_open);
       break;
     case PROP_PIXBUF_EXPANDER_CLOSED:
-      g_value_set_object (value, cellpixbuf->pixbuf_expander_closed);
+      g_value_set_object (value, priv->pixbuf_expander_closed);
       break;
     case PROP_STOCK_ID:
       g_value_set_string (value, priv->stock_id);
@@ -290,17 +295,15 @@ gtk_cell_renderer_pixbuf_set_property (GObject      *object,
 				       GParamSpec   *pspec)
 {
   GtkCellRendererPixbuf *cellpixbuf = GTK_CELL_RENDERER_PIXBUF (object);
-  GtkCellRendererPixbufPrivate *priv;
+  GtkCellRendererPixbufPrivate *priv = cellpixbuf->priv;
 
-  priv = GTK_CELL_RENDERER_PIXBUF_GET_PRIVATE (object);
-  
   switch (param_id)
     {
     case PROP_PIXBUF:
-      if (cellpixbuf->pixbuf)
-	g_object_unref (cellpixbuf->pixbuf);
-      cellpixbuf->pixbuf = (GdkPixbuf*) g_value_dup_object (value);
-      if (cellpixbuf->pixbuf)
+      if (priv->pixbuf)
+        g_object_unref (priv->pixbuf);
+      priv->pixbuf = (GdkPixbuf*) g_value_dup_object (value);
+      if (priv->pixbuf)
         {
           if (priv->stock_id)
             {
@@ -323,22 +326,22 @@ gtk_cell_renderer_pixbuf_set_property (GObject      *object,
         }
       break;
     case PROP_PIXBUF_EXPANDER_OPEN:
-      if (cellpixbuf->pixbuf_expander_open)
-	g_object_unref (cellpixbuf->pixbuf_expander_open);
-      cellpixbuf->pixbuf_expander_open = (GdkPixbuf*) g_value_dup_object (value);
+      if (priv->pixbuf_expander_open)
+        g_object_unref (priv->pixbuf_expander_open);
+      priv->pixbuf_expander_open = (GdkPixbuf*) g_value_dup_object (value);
       break;
     case PROP_PIXBUF_EXPANDER_CLOSED:
-      if (cellpixbuf->pixbuf_expander_closed)
-	g_object_unref (cellpixbuf->pixbuf_expander_closed);
-      cellpixbuf->pixbuf_expander_closed = (GdkPixbuf*) g_value_dup_object (value);
+      if (priv->pixbuf_expander_closed)
+        g_object_unref (priv->pixbuf_expander_closed);
+      priv->pixbuf_expander_closed = (GdkPixbuf*) g_value_dup_object (value);
       break;
     case PROP_STOCK_ID:
       if (priv->stock_id)
         {
-          if (cellpixbuf->pixbuf)
+          if (priv->pixbuf)
             {
-              g_object_unref (cellpixbuf->pixbuf);
-              cellpixbuf->pixbuf = NULL;
+              g_object_unref (priv->pixbuf);
+              priv->pixbuf = NULL;
               g_object_notify (object, "pixbuf");
             }
           g_free (priv->stock_id);
@@ -346,10 +349,10 @@ gtk_cell_renderer_pixbuf_set_property (GObject      *object,
       priv->stock_id = g_value_dup_string (value);
       if (priv->stock_id)
         {
-          if (cellpixbuf->pixbuf)
+          if (priv->pixbuf)
             {
-              g_object_unref (cellpixbuf->pixbuf);
-              cellpixbuf->pixbuf = NULL;
+              g_object_unref (priv->pixbuf);
+              priv->pixbuf = NULL;
               g_object_notify (object, "pixbuf");
             }
           if (priv->icon_name)
@@ -375,24 +378,24 @@ gtk_cell_renderer_pixbuf_set_property (GObject      *object,
       break;
     case PROP_ICON_NAME:
       if (priv->icon_name)
-	{
-	  if (cellpixbuf->pixbuf)
-	    {
-	      g_object_unref (cellpixbuf->pixbuf);
-	      cellpixbuf->pixbuf = NULL;
+        {
+          if (priv->pixbuf)
+            {
+              g_object_unref (priv->pixbuf);
+              priv->pixbuf = NULL;
               g_object_notify (object, "pixbuf");
-	    }
-	  g_free (priv->icon_name);
-	}
+            }
+          g_free (priv->icon_name);
+        }
       priv->icon_name = g_value_dup_string (value);
       if (priv->icon_name)
         {
-	  if (cellpixbuf->pixbuf)
-	    {
-              g_object_unref (cellpixbuf->pixbuf);
-              cellpixbuf->pixbuf = NULL;
+          if (priv->pixbuf)
+            {
+              g_object_unref (priv->pixbuf);
+              priv->pixbuf = NULL;
               g_object_notify (object, "pixbuf");
-	    }
+            }
           if (priv->stock_id)
             {
               g_free (priv->stock_id);
@@ -412,24 +415,24 @@ gtk_cell_renderer_pixbuf_set_property (GObject      *object,
       break;
     case PROP_GICON:
       if (priv->gicon)
-	{
-	  if (cellpixbuf->pixbuf)
-	    {
-	      g_object_unref (cellpixbuf->pixbuf);
-	      cellpixbuf->pixbuf = NULL;
+        {
+          if (priv->pixbuf)
+            {
+              g_object_unref (priv->pixbuf);
+              priv->pixbuf = NULL;
               g_object_notify (object, "pixbuf");
-	    }
-	  g_object_unref (priv->gicon);
-	}
+            }
+          g_object_unref (priv->gicon);
+        }
       priv->gicon = (GIcon *) g_value_dup_object (value);
       if (priv->gicon)
         {
-	  if (cellpixbuf->pixbuf)
-	    {
-              g_object_unref (cellpixbuf->pixbuf);
-              cellpixbuf->pixbuf = NULL;
+          if (priv->pixbuf)
+            {
+              g_object_unref (priv->pixbuf);
+              priv->pixbuf = NULL;
               g_object_notify (object, "pixbuf");
-	    }
+            }
           if (priv->stock_id)
             {
               g_free (priv->stock_id);
@@ -473,17 +476,14 @@ static void
 gtk_cell_renderer_pixbuf_create_stock_pixbuf (GtkCellRendererPixbuf *cellpixbuf,
                                               GtkWidget             *widget)
 {
-  GtkCellRendererPixbufPrivate *priv;
+  GtkCellRendererPixbufPrivate *priv = cellpixbuf->priv;
 
-  priv = GTK_CELL_RENDERER_PIXBUF_GET_PRIVATE (cellpixbuf);
+  if (priv->pixbuf)
+    g_object_unref (priv->pixbuf);
 
-  if (cellpixbuf->pixbuf)
-    g_object_unref (cellpixbuf->pixbuf);
-
-  cellpixbuf->pixbuf = gtk_widget_render_icon (widget,
-                                               priv->stock_id,
-                                               priv->stock_size,
-                                               priv->stock_detail);
+  priv->pixbuf = gtk_widget_render_icon_pixbuf (widget,
+                                                priv->stock_id,
+                                                priv->stock_size);
 
   g_object_notify (G_OBJECT (cellpixbuf), "pixbuf");
 }
@@ -492,18 +492,17 @@ static void
 gtk_cell_renderer_pixbuf_create_themed_pixbuf (GtkCellRendererPixbuf *cellpixbuf,
 					       GtkWidget             *widget)
 {
-  GtkCellRendererPixbufPrivate *priv;
+  GtkCellRendererPixbufPrivate *priv = cellpixbuf->priv;
   GdkScreen *screen;
   GtkIconTheme *icon_theme;
   GtkSettings *settings;
   gint width, height;
+  GtkIconInfo *info;
 
-  priv = GTK_CELL_RENDERER_PIXBUF_GET_PRIVATE (cellpixbuf);
-
-  if (cellpixbuf->pixbuf)
+  if (priv->pixbuf)
     {
-      g_object_unref (cellpixbuf->pixbuf);
-      cellpixbuf->pixbuf = NULL;
+      g_object_unref (priv->pixbuf);
+      priv->pixbuf = NULL;
     }
 
   screen = gtk_widget_get_screen (GTK_WIDGET (widget));
@@ -519,32 +518,112 @@ gtk_cell_renderer_pixbuf_create_themed_pixbuf (GtkCellRendererPixbuf *cellpixbuf
     }
 
   if (priv->icon_name)
-    cellpixbuf->pixbuf = gtk_icon_theme_load_icon (icon_theme,
-			                           priv->icon_name,
-			                           MIN (width, height), 
-                                                   GTK_ICON_LOOKUP_USE_BUILTIN,
-                                                   NULL);
+    info = gtk_icon_theme_lookup_icon (icon_theme,
+                                       priv->icon_name,
+                                       MIN (width, height),
+                                       GTK_ICON_LOOKUP_USE_BUILTIN);
   else if (priv->gicon)
-    {
-      GtkIconInfo *info;
+    info = gtk_icon_theme_lookup_by_gicon (icon_theme,
+                                           priv->gicon,
+                                           MIN (width, height),
+                                           GTK_ICON_LOOKUP_USE_BUILTIN);
+  else
+    info = NULL;
 
-      info = gtk_icon_theme_lookup_by_gicon (icon_theme,
-                                             priv->gicon,
-			                     MIN (width, height), 
-                                             GTK_ICON_LOOKUP_USE_BUILTIN);
-      if (info)
-        {
-          cellpixbuf->pixbuf = gtk_icon_info_load_icon (info, NULL);
-          gtk_icon_info_free (info);
-        }
+  if (info)
+    {
+      GtkStyleContext *context;
+
+      context = gtk_widget_get_style_context (GTK_WIDGET (widget));
+      priv->pixbuf = gtk_icon_info_load_symbolic_for_context (info,
+                                                              context,
+                                                              NULL,
+                                                              NULL);
+      gtk_icon_info_free (info);
     }
 
   g_object_notify (G_OBJECT (cellpixbuf), "pixbuf");
 }
 
 static GdkPixbuf *
-create_colorized_pixbuf (GdkPixbuf *src, 
-			 GdkColor  *new_color)
+create_symbolic_pixbuf (GtkCellRendererPixbuf *cellpixbuf,
+			GtkWidget             *widget,
+                        GtkStateFlags          state)
+{
+  GtkCellRendererPixbufPrivate *priv = cellpixbuf->priv;
+  GdkScreen *screen;
+  GtkIconTheme *icon_theme;
+  GtkSettings *settings;
+  gint width, height;
+  GtkIconInfo *info;
+  GdkPixbuf *pixbuf;
+
+  /* Not a named symbolic icon? */
+  if (priv->icon_name) {
+    if (!g_str_has_suffix (priv->icon_name, "-symbolic"))
+      return NULL;
+  } else if (priv->gicon) {
+    const gchar * const *names;
+    if (!G_IS_THEMED_ICON (priv->gicon))
+      return NULL;
+    names = g_themed_icon_get_names (G_THEMED_ICON (priv->gicon));
+    if (names == NULL || !g_str_has_suffix (names[0], "-symbolic"))
+      return NULL;
+  } else {
+    return NULL;
+  }
+
+  screen = gtk_widget_get_screen (GTK_WIDGET (widget));
+  icon_theme = gtk_icon_theme_get_for_screen (screen);
+  settings = gtk_settings_get_for_screen (screen);
+
+  if (!gtk_icon_size_lookup_for_settings (settings,
+					  priv->stock_size,
+					  &width, &height))
+    {
+      g_warning ("Invalid icon size %u\n", priv->stock_size);
+      width = height = 24;
+    }
+
+
+  if (priv->icon_name)
+    info = gtk_icon_theme_lookup_icon (icon_theme,
+                                       priv->icon_name,
+                                       MIN (width, height),
+                                       GTK_ICON_LOOKUP_USE_BUILTIN);
+  else if (priv->gicon)
+    info = gtk_icon_theme_lookup_by_gicon (icon_theme,
+                                           priv->gicon,
+                                           MIN (width, height),
+                                           GTK_ICON_LOOKUP_USE_BUILTIN);
+  else
+    return NULL;
+
+  if (info)
+    {
+      GtkStyleContext *context;
+
+      context = gtk_widget_get_style_context (GTK_WIDGET (widget));
+
+      gtk_style_context_save (context);
+      gtk_style_context_set_state (context, state);
+      pixbuf = gtk_icon_info_load_symbolic_for_context (info,
+                                                        context,
+                                                        NULL,
+                                                        NULL);
+
+      gtk_style_context_restore (context);
+      gtk_icon_info_free (info);
+
+      return pixbuf;
+    }
+
+  return NULL;
+}
+
+static GdkPixbuf *
+create_colorized_pixbuf (GdkPixbuf *src,
+                         GdkRGBA   *new_color)
 {
   gint i, j;
   gint width, height, has_alpha, src_row_stride, dst_row_stride;
@@ -554,11 +633,11 @@ create_colorized_pixbuf (GdkPixbuf *src,
   guchar *pixsrc;
   guchar *pixdest;
   GdkPixbuf *dest;
-  
-  red_value = new_color->red / 255.0;
-  green_value = new_color->green / 255.0;
-  blue_value = new_color->blue / 255.0;
-  
+
+  red_value = (new_color->red * 65535.0) / 255.0;
+  green_value = (new_color->green * 65535.0) / 255.0;
+  blue_value = (new_color->blue * 65535.0) / 255.0;
+
   dest = gdk_pixbuf_new (gdk_pixbuf_get_colorspace (src),
 			 gdk_pixbuf_get_has_alpha (src),
 			 gdk_pixbuf_get_bits_per_sample (src),
@@ -590,24 +669,23 @@ create_colorized_pixbuf (GdkPixbuf *src,
 
 
 static void
-gtk_cell_renderer_pixbuf_get_size (GtkCellRenderer *cell,
-				   GtkWidget       *widget,
-				   GdkRectangle    *cell_area,
-				   gint            *x_offset,
-				   gint            *y_offset,
-				   gint            *width,
-				   gint            *height)
+gtk_cell_renderer_pixbuf_get_size (GtkCellRenderer    *cell,
+				   GtkWidget          *widget,
+				   const GdkRectangle *cell_area,
+				   gint               *x_offset,
+				   gint               *y_offset,
+				   gint               *width,
+				   gint               *height)
 {
   GtkCellRendererPixbuf *cellpixbuf = (GtkCellRendererPixbuf *) cell;
-  GtkCellRendererPixbufPrivate *priv;
+  GtkCellRendererPixbufPrivate *priv = cellpixbuf->priv;
   gint pixbuf_width  = 0;
   gint pixbuf_height = 0;
   gint calc_width;
   gint calc_height;
+  gint xpad, ypad;
 
-  priv = GTK_CELL_RENDERER_PIXBUF_GET_PRIVATE (cell);
-
-  if (!cellpixbuf->pixbuf)
+  if (!priv->pixbuf)
     {
       if (priv->stock_id)
 	gtk_cell_renderer_pixbuf_create_stock_pixbuf (cellpixbuf, widget);
@@ -615,37 +693,41 @@ gtk_cell_renderer_pixbuf_get_size (GtkCellRenderer *cell,
 	gtk_cell_renderer_pixbuf_create_themed_pixbuf (cellpixbuf, widget);
     }
   
-  if (cellpixbuf->pixbuf)
+  if (priv->pixbuf)
     {
-      pixbuf_width  = gdk_pixbuf_get_width (cellpixbuf->pixbuf);
-      pixbuf_height = gdk_pixbuf_get_height (cellpixbuf->pixbuf);
+      pixbuf_width  = gdk_pixbuf_get_width (priv->pixbuf);
+      pixbuf_height = gdk_pixbuf_get_height (priv->pixbuf);
     }
-  if (cellpixbuf->pixbuf_expander_open)
+  if (priv->pixbuf_expander_open)
     {
-      pixbuf_width  = MAX (pixbuf_width, gdk_pixbuf_get_width (cellpixbuf->pixbuf_expander_open));
-      pixbuf_height = MAX (pixbuf_height, gdk_pixbuf_get_height (cellpixbuf->pixbuf_expander_open));
+      pixbuf_width  = MAX (pixbuf_width, gdk_pixbuf_get_width (priv->pixbuf_expander_open));
+      pixbuf_height = MAX (pixbuf_height, gdk_pixbuf_get_height (priv->pixbuf_expander_open));
     }
-  if (cellpixbuf->pixbuf_expander_closed)
+  if (priv->pixbuf_expander_closed)
     {
-      pixbuf_width  = MAX (pixbuf_width, gdk_pixbuf_get_width (cellpixbuf->pixbuf_expander_closed));
-      pixbuf_height = MAX (pixbuf_height, gdk_pixbuf_get_height (cellpixbuf->pixbuf_expander_closed));
+      pixbuf_width  = MAX (pixbuf_width, gdk_pixbuf_get_width (priv->pixbuf_expander_closed));
+      pixbuf_height = MAX (pixbuf_height, gdk_pixbuf_get_height (priv->pixbuf_expander_closed));
     }
-  
-  calc_width  = (gint) cell->xpad * 2 + pixbuf_width;
-  calc_height = (gint) cell->ypad * 2 + pixbuf_height;
+
+  gtk_cell_renderer_get_padding (cell, &xpad, &ypad);
+  calc_width  = (gint) xpad * 2 + pixbuf_width;
+  calc_height = (gint) ypad * 2 + pixbuf_height;
   
   if (cell_area && pixbuf_width > 0 && pixbuf_height > 0)
     {
+      gfloat xalign, yalign;
+
+      gtk_cell_renderer_get_alignment (cell, &xalign, &yalign);
       if (x_offset)
 	{
 	  *x_offset = (((gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL) ?
-                        (1.0 - cell->xalign) : cell->xalign) * 
+                        (1.0 - xalign) : xalign) *
                        (cell_area->width - calc_width));
 	  *x_offset = MAX (*x_offset, 0);
 	}
       if (y_offset)
 	{
-	  *y_offset = (cell->yalign *
+	  *y_offset = (yalign *
                        (cell_area->height - calc_height));
           *y_offset = MAX (*y_offset, 0);
 	}
@@ -665,56 +747,64 @@ gtk_cell_renderer_pixbuf_get_size (GtkCellRenderer *cell,
 
 static void
 gtk_cell_renderer_pixbuf_render (GtkCellRenderer      *cell,
-				 GdkWindow            *window,
+                                 cairo_t              *cr,
 				 GtkWidget            *widget,
-				 GdkRectangle         *background_area,
-				 GdkRectangle         *cell_area,
-				 GdkRectangle         *expose_area,
+				 const GdkRectangle   *background_area,
+				 const GdkRectangle   *cell_area,
 				 GtkCellRendererState  flags)
 
 {
   GtkCellRendererPixbuf *cellpixbuf = (GtkCellRendererPixbuf *) cell;
-  GtkCellRendererPixbufPrivate *priv;
+  GtkCellRendererPixbufPrivate *priv = cellpixbuf->priv;
+  GtkStyleContext *context;
   GdkPixbuf *pixbuf;
   GdkPixbuf *invisible = NULL;
   GdkPixbuf *colorized = NULL;
+  GdkPixbuf *symbolic = NULL;
   GdkRectangle pix_rect;
   GdkRectangle draw_rect;
-  cairo_t *cr;
+  gboolean is_expander;
+  gint xpad, ypad;
 
-  priv = GTK_CELL_RENDERER_PIXBUF_GET_PRIVATE (cell);
-
-  gtk_cell_renderer_pixbuf_get_size (cell, widget, cell_area,
+  gtk_cell_renderer_pixbuf_get_size (cell, widget, (GdkRectangle *) cell_area,
 				     &pix_rect.x,
 				     &pix_rect.y,
 				     &pix_rect.width,
 				     &pix_rect.height);
 
-  pix_rect.x += cell_area->x + cell->xpad;
-  pix_rect.y += cell_area->y + cell->ypad;
-  pix_rect.width  -= cell->xpad * 2;
-  pix_rect.height -= cell->ypad * 2;
+  gtk_cell_renderer_get_padding (cell, &xpad, &ypad);
+  pix_rect.x += cell_area->x + xpad;
+  pix_rect.y += cell_area->y + ypad;
+  pix_rect.width  -= xpad * 2;
+  pix_rect.height -= ypad * 2;
 
-  if (!gdk_rectangle_intersect (cell_area, &pix_rect, &draw_rect) ||
-      !gdk_rectangle_intersect (expose_area, &draw_rect, &draw_rect))
+  if (!gdk_rectangle_intersect (cell_area, &pix_rect, &draw_rect))
     return;
 
-  pixbuf = cellpixbuf->pixbuf;
+  pixbuf = priv->pixbuf;
 
-  if (cell->is_expander)
+  g_object_get (cell, "is-expander", &is_expander, NULL);
+  if (is_expander)
     {
-      if (cell->is_expanded &&
-	  cellpixbuf->pixbuf_expander_open != NULL)
-	pixbuf = cellpixbuf->pixbuf_expander_open;
-      else if (!cell->is_expanded &&
-	       cellpixbuf->pixbuf_expander_closed != NULL)
-	pixbuf = cellpixbuf->pixbuf_expander_closed;
+      gboolean is_expanded;
+
+      g_object_get (cell, "is-expanded", &is_expanded, NULL);
+
+      if (is_expanded &&
+	  priv->pixbuf_expander_open != NULL)
+	pixbuf = priv->pixbuf_expander_open;
+      else if (!is_expanded &&
+	       priv->pixbuf_expander_closed != NULL)
+	pixbuf = priv->pixbuf_expander_closed;
     }
 
   if (!pixbuf)
     return;
 
-  if (GTK_WIDGET_STATE (widget) == GTK_STATE_INSENSITIVE || !cell->sensitive)
+  context = gtk_widget_get_style_context (widget);
+
+  if (!gtk_widget_get_sensitive (widget) ||
+      !gtk_cell_renderer_get_sensitive (cell))
     {
       GtkIconSource *source;
       
@@ -726,55 +816,45 @@ gtk_cell_renderer_pixbuf_render (GtkCellRenderer      *cell,
        */
       gtk_icon_source_set_size (source, GTK_ICON_SIZE_SMALL_TOOLBAR);
       gtk_icon_source_set_size_wildcarded (source, FALSE);
-      
-     invisible = gtk_style_render_icon (widget->style,
-					source,
-					gtk_widget_get_direction (widget),
-					GTK_STATE_INSENSITIVE,
-					/* arbitrary */
-					(GtkIconSize)-1,
-					widget,
-					"gtkcellrendererpixbuf");
-     
-     gtk_icon_source_free (source);
-     
-     pixbuf = invisible;
+
+      gtk_style_context_save (context);
+      gtk_style_context_set_state (context, GTK_STATE_FLAG_INSENSITIVE);
+
+      pixbuf = invisible = gtk_render_icon_pixbuf (context, source,
+                                                   (GtkIconSize) -1);
+
+      gtk_style_context_restore (context);
+      gtk_icon_source_free (source);
     }
   else if (priv->follow_state && 
 	   (flags & (GTK_CELL_RENDERER_SELECTED|GTK_CELL_RENDERER_PRELIT)) != 0)
     {
-      GtkStateType state;
+      GtkStateFlags state;
 
-      if ((flags & GTK_CELL_RENDERER_SELECTED) != 0)
-	{
-	  if (GTK_WIDGET_HAS_FOCUS (widget))
-	    state = GTK_STATE_SELECTED;
-	  else
-	    state = GTK_STATE_ACTIVE;
-	}
+      state = gtk_cell_renderer_get_state (cell, widget, flags);
+      symbolic = create_symbolic_pixbuf (cellpixbuf, widget, state);
+
+      if (!symbolic)
+        {
+          GdkRGBA color;
+
+          gtk_style_context_get_background_color (context, state, &color);
+          pixbuf = colorized = create_colorized_pixbuf (pixbuf, &color);
+        }
       else
-	state = GTK_STATE_PRELIGHT;
-
-      colorized = create_colorized_pixbuf (pixbuf,
-					   &widget->style->base[state]);
-
-      pixbuf = colorized;
+        pixbuf = symbolic;
     }
 
-  cr = gdk_cairo_create (window);
-  
   gdk_cairo_set_source_pixbuf (cr, pixbuf, pix_rect.x, pix_rect.y);
   gdk_cairo_rectangle (cr, &draw_rect);
   cairo_fill (cr);
 
-  cairo_destroy (cr);
-  
   if (invisible)
     g_object_unref (invisible);
 
   if (colorized)
     g_object_unref (colorized);
-}
 
-#define __GTK_CELL_RENDERER_PIXBUF_C__
-#include "gtkaliasdef.c"
+  if (symbolic)
+    g_object_unref (symbolic);
+}
