@@ -18,7 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
 #error "Only <gtk/gtk.h> can be included directly."
 #endif
 
@@ -43,10 +43,10 @@ typedef struct _GtkFileChooserButtonClass   GtkFileChooserButtonClass;
 
 struct _GtkFileChooserButton
 {
-  /*< private >*/
   GtkHBox parent;
 
-  GtkFileChooserButtonPrivate *GSEAL (priv);
+  /*< private >*/
+  GtkFileChooserButtonPrivate *priv;
 };
 
 struct _GtkFileChooserButtonClass
@@ -56,22 +56,17 @@ struct _GtkFileChooserButtonClass
 
   void (* file_set) (GtkFileChooserButton *fc);
 
+  /* Padding for future expansion */
   void (*__gtk_reserved1);
   void (*__gtk_reserved2);
   void (*__gtk_reserved3);
   void (*__gtk_reserved4);
-  void (*__gtk_reserved5);
-  void (*__gtk_reserved6);
-  void (*__gtk_reserved7);
 };
 
 
 GType                 gtk_file_chooser_button_get_type         (void) G_GNUC_CONST;
 GtkWidget *           gtk_file_chooser_button_new              (const gchar          *title,
 								GtkFileChooserAction  action);
-GtkWidget *           gtk_file_chooser_button_new_with_backend (const gchar          *title,
-								GtkFileChooserAction  action,
-								const gchar          *backend);
 GtkWidget *           gtk_file_chooser_button_new_with_dialog  (GtkWidget            *dialog);
 G_CONST_RETURN gchar *gtk_file_chooser_button_get_title        (GtkFileChooserButton *button);
 void                  gtk_file_chooser_button_set_title        (GtkFileChooserButton *button,
