@@ -73,8 +73,7 @@ static void gtk_tool_button_toolbar_reconfigured (GtkToolItem *tool_item);
 static gboolean   gtk_tool_button_create_menu_proxy (GtkToolItem     *item);
 static void       button_clicked                    (GtkWidget       *widget,
 						     GtkToolButton   *button);
-static void gtk_tool_button_style_set      (GtkWidget          *widget,
-					    GtkStyle           *prev_style);
+static void gtk_tool_button_style_updated  (GtkWidget          *widget);
 
 static void gtk_tool_button_construct_contents (GtkToolItem *tool_item);
 
@@ -106,7 +105,6 @@ static GObjectClass        *parent_class = NULL;
 static GtkActivatableIface *parent_activatable_iface;
 static guint                toolbutton_signals[LAST_SIGNAL] = { 0 };
 
-#define GTK_TOOL_BUTTON_GET_PRIVATE(obj)(G_TYPE_INSTANCE_GET_PRIVATE ((obj), GTK_TYPE_TOOL_BUTTON, GtkToolButtonPrivate))
 
 GType
 gtk_tool_button_get_type (void)
@@ -154,7 +152,7 @@ gtk_tool_button_class_init (GtkToolButtonClass *klass)
   object_class->notify = gtk_tool_button_property_notify;
   object_class->finalize = gtk_tool_button_finalize;
 
-  widget_class->style_set = gtk_tool_button_style_set;
+  widget_class->style_updated = gtk_tool_button_style_updated;
 
   tool_item_class->create_menu_proxy = gtk_tool_button_create_menu_proxy;
   tool_item_class->toolbar_reconfigured = gtk_tool_button_toolbar_reconfigured;
@@ -290,8 +288,10 @@ gtk_tool_button_init (GtkToolButton      *button,
 		      GtkToolButtonClass *klass)
 {
   GtkToolItem *toolitem = GTK_TOOL_ITEM (button);
-  
-  button->priv = GTK_TOOL_BUTTON_GET_PRIVATE (button);
+
+  button->priv = G_TYPE_INSTANCE_GET_PRIVATE (button,
+                                              GTK_TYPE_TOOL_BUTTON,
+                                              GtkToolButtonPrivate);
 
   button->priv->contents_invalid = TRUE;
 
@@ -311,6 +311,7 @@ static void
 gtk_tool_button_construct_contents (GtkToolItem *tool_item)
 {
   GtkToolButton *button = GTK_TOOL_BUTTON (tool_item);
+  GtkWidget *child;
   GtkWidget *label = NULL;
   GtkWidget *icon = NULL;
   GtkToolbarStyle style;
@@ -321,6 +322,7 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
   guint icon_spacing;
   GtkOrientation text_orientation = GTK_ORIENTATION_HORIZONTAL;
   GtkSizeGroup *size_group = NULL;
+  GtkWidget *parent;
 
   button->priv->contents_invalid = FALSE;
 
@@ -328,24 +330,33 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
 			"icon-spacing", &icon_spacing,
 			NULL);
 
-  if (button->priv->icon_widget && button->priv->icon_widget->parent)
+  if (button->priv->icon_widget)
     {
-      gtk_container_remove (GTK_CONTAINER (button->priv->icon_widget->parent),
-			    button->priv->icon_widget);
+      parent = gtk_widget_get_parent (button->priv->icon_widget);
+      if (parent)
+        {
+          gtk_container_remove (GTK_CONTAINER (parent),
+                                button->priv->icon_widget);
+        }
     }
 
-  if (button->priv->label_widget && button->priv->label_widget->parent)
+  if (button->priv->label_widget)
     {
-      gtk_container_remove (GTK_CONTAINER (button->priv->label_widget->parent),
-			    button->priv->label_widget);
+      parent = gtk_widget_get_parent (button->priv->label_widget);
+      if (parent)
+        {
+          gtk_container_remove (GTK_CONTAINER (parent),
+                                button->priv->label_widget);
+        }
     }
 
-  if (GTK_BIN (button->priv->button)->child)
+  child = gtk_bin_get_child (GTK_BIN (button->priv->button));
+  if (child)
     {
       /* Note: we are not destroying the label_widget or icon_widget
        * here because they were removed from their containers above
        */
-      gtk_widget_destroy (GTK_BIN (button->priv->button)->child);
+      gtk_widget_destroy (child);
     }
 
   style = gtk_tool_item_get_toolbar_style (GTK_TOOL_ITEM (button));
@@ -420,26 +431,29 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
 	  gtk_widget_show (label);
 	}
 
-      gtk_label_set_ellipsize (GTK_LABEL (label),
-			       gtk_tool_item_get_ellipsize_mode (GTK_TOOL_ITEM (button)));
-      text_orientation = gtk_tool_item_get_text_orientation (GTK_TOOL_ITEM (button));
-      if (text_orientation == GTK_ORIENTATION_HORIZONTAL)
-	{
-          gtk_label_set_angle (GTK_LABEL (label), 0);
-          gtk_misc_set_alignment (GTK_MISC (label),
-                                  gtk_tool_item_get_text_alignment (GTK_TOOL_ITEM (button)),
-                                  0.5);
-        }
-      else
+      if (GTK_IS_LABEL (label))
         {
-          gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_NONE);
-	  if (gtk_widget_get_direction (GTK_WIDGET (tool_item)) == GTK_TEXT_DIR_RTL)
-	    gtk_label_set_angle (GTK_LABEL (label), -90);
-	  else
-	    gtk_label_set_angle (GTK_LABEL (label), 90);
-          gtk_misc_set_alignment (GTK_MISC (label),
-                                  0.5,
-                                  1 - gtk_tool_item_get_text_alignment (GTK_TOOL_ITEM (button)));
+          gtk_label_set_ellipsize (GTK_LABEL (label),
+			           gtk_tool_item_get_ellipsize_mode (GTK_TOOL_ITEM (button)));
+          text_orientation = gtk_tool_item_get_text_orientation (GTK_TOOL_ITEM (button));
+          if (text_orientation == GTK_ORIENTATION_HORIZONTAL)
+	    {
+              gtk_label_set_angle (GTK_LABEL (label), 0);
+              gtk_misc_set_alignment (GTK_MISC (label),
+                                      gtk_tool_item_get_text_alignment (GTK_TOOL_ITEM (button)),
+                                      0.5);
+            }
+          else
+            {
+              gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_NONE);
+	      if (gtk_widget_get_direction (GTK_WIDGET (tool_item)) == GTK_TEXT_DIR_RTL)
+	        gtk_label_set_angle (GTK_LABEL (label), -90);
+	      else
+	        gtk_label_set_angle (GTK_LABEL (label), 90);
+              gtk_misc_set_alignment (GTK_MISC (label),
+                                      0.5,
+                                      1 - gtk_tool_item_get_text_alignment (GTK_TOOL_ITEM (button)));
+            }
         }
     }
 
@@ -495,9 +509,9 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
 
     case GTK_TOOLBAR_BOTH:
       if (text_orientation == GTK_ORIENTATION_HORIZONTAL)
-	box = gtk_vbox_new (FALSE, icon_spacing);
+	box = gtk_box_new (GTK_ORIENTATION_VERTICAL, icon_spacing);
       else
-	box = gtk_hbox_new (FALSE, icon_spacing);
+	box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, icon_spacing);
       if (icon)
 	gtk_box_pack_start (GTK_BOX (box), icon, TRUE, TRUE, 0);
       gtk_box_pack_end (GTK_BOX (box), label, FALSE, TRUE, 0);
@@ -507,7 +521,7 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
     case GTK_TOOLBAR_BOTH_HORIZ:
       if (text_orientation == GTK_ORIENTATION_HORIZONTAL)
 	{
-	  box = gtk_hbox_new (FALSE, icon_spacing);
+	  box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, icon_spacing);
 	  if (icon)
 	    gtk_box_pack_start (GTK_BOX (box), icon, label? FALSE : TRUE, TRUE, 0);
 	  if (label)
@@ -515,7 +529,7 @@ gtk_tool_button_construct_contents (GtkToolItem *tool_item)
 	}
       else
 	{
-	  box = gtk_vbox_new (FALSE, icon_spacing);
+	  box = gtk_box_new (GTK_ORIENTATION_VERTICAL, icon_spacing);
 	  if (icon)
 	    gtk_box_pack_end (GTK_BOX (box), icon, label ? FALSE : TRUE, TRUE, 0);
 	  if (label)
@@ -781,7 +795,7 @@ gtk_tool_button_update_icon_spacing (GtkToolButton *button)
   GtkWidget *box;
   guint spacing;
 
-  box = GTK_BIN (button->priv->button)->child;
+  box = gtk_bin_get_child (GTK_BIN (button->priv->button));
   if (GTK_IS_BOX (box))
     {
       gtk_widget_style_get (GTK_WIDGET (button), 
@@ -792,8 +806,7 @@ gtk_tool_button_update_icon_spacing (GtkToolButton *button)
 }
 
 static void
-gtk_tool_button_style_set (GtkWidget *widget,
-			   GtkStyle  *prev_style)
+gtk_tool_button_style_updated (GtkWidget *widget)
 {
   gtk_tool_button_update_icon_spacing (GTK_TOOL_BUTTON (widget));
 }
@@ -1187,9 +1200,12 @@ gtk_tool_button_set_icon_widget (GtkToolButton *button,
     {
       if (button->priv->icon_widget)
 	{
-	  if (button->priv->icon_widget->parent)
-	    gtk_container_remove (GTK_CONTAINER (button->priv->icon_widget->parent),
-				    button->priv->icon_widget);
+          GtkWidget *parent;
+
+          parent = gtk_widget_get_parent (button->priv->icon_widget);
+	  if (parent)
+            gtk_container_remove (GTK_CONTAINER (parent),
+                                  button->priv->icon_widget);
 
 	  g_object_unref (button->priv->icon_widget);
 	}
@@ -1228,10 +1244,13 @@ gtk_tool_button_set_label_widget (GtkToolButton *button,
     {
       if (button->priv->label_widget)
 	{
-	  if (button->priv->label_widget->parent)
-	    gtk_container_remove (GTK_CONTAINER (button->priv->label_widget->parent),
-		    	          button->priv->label_widget);
-	  
+          GtkWidget *parent;
+
+          parent = gtk_widget_get_parent (button->priv->label_widget);
+          if (parent)
+            gtk_container_remove (GTK_CONTAINER (parent),
+                                  button->priv->label_widget);
+
 	  g_object_unref (button->priv->label_widget);
 	}
       
@@ -1248,12 +1267,13 @@ gtk_tool_button_set_label_widget (GtkToolButton *button,
 /**
  * gtk_tool_button_get_label_widget:
  * @button: a #GtkToolButton
- * 
- * Returns the widget used as label on @button. See
- * gtk_tool_button_set_label_widget().
- * 
- * Return value: The widget used as label on @button, or %NULL.
- * 
+ *
+ * Returns the widget used as label on @button.
+ * See gtk_tool_button_set_label_widget().
+ *
+ * Return value: (transfer none): The widget used as label
+ *     on @button, or %NULL.
+ *
  * Since: 2.4
  **/
 GtkWidget *
@@ -1267,12 +1287,13 @@ gtk_tool_button_get_label_widget (GtkToolButton *button)
 /**
  * gtk_tool_button_get_icon_widget:
  * @button: a #GtkToolButton
- * 
- * Return the widget used as icon widget on @button. See
- * gtk_tool_button_set_icon_widget().
- * 
- * Return value: The widget used as icon on @button, or %NULL.
- * 
+ *
+ * Return the widget used as icon widget on @button.
+ * See gtk_tool_button_set_icon_widget().
+ *
+ * Return value: (transfer none): The widget used as icon
+ *     on @button, or %NULL.
+ *
  * Since: 2.4
  **/
 GtkWidget *
