@@ -20,9 +20,8 @@
  * do not use it.
  */
 
-#include <config.h>
+#include "config.h"
 #include "gtktreedatalist.h"
-#include "gtkalias.h"
 #include <string.h>
 
 /* node allocation
@@ -483,7 +482,7 @@ _gtk_tree_data_list_header_free (GList *list)
 
       if (header->destroy)
 	{
-	  GtkDestroyNotify d = header->destroy;
+	  GDestroyNotify d = header->destroy;
 
 	  header->destroy = NULL;
 	  d (header->data);
@@ -515,7 +514,7 @@ _gtk_tree_data_list_set_header (GList                  *header_list,
 				gint                    sort_column_id,
 				GtkTreeIterCompareFunc  func,
 				gpointer                data,
-				GtkDestroyNotify        destroy)
+				GDestroyNotify          destroy)
 {
   GList *list = header_list;
   GtkTreeDataSortHeader *header = NULL;
@@ -543,7 +542,7 @@ _gtk_tree_data_list_set_header (GList                  *header_list,
 
   if (header->destroy)
     {
-      GtkDestroyNotify d = header->destroy;
+      GDestroyNotify d = header->destroy;
       
       header->destroy = NULL;
       d (header->data);
