@@ -76,6 +76,9 @@ static GObject *gtk_tree_view_column_constructor               (GType
 								GObjectConstructParam   *construct_properties);
 
 /* GtkCellLayout implementation */
+static void       gtk_tree_view_column_ensure_cell_area        (GtkTreeViewColumn      *column,
+                                                                GtkCellArea            *cell_area);
+
 static GtkCellArea *gtk_tree_view_column_cell_layout_get_area  (GtkCellLayout           *cell_layout);
 
 /* Button handling code */
@@ -476,42 +479,19 @@ gtk_tree_view_column_init (GtkTreeViewColumn *tree_column)
 }
 
 static GObject *
-gtk_tree_view_column_constructor (GType                    type,
-				  guint                    n_construct_properties,
-				  GObjectConstructParam   *construct_properties)
+gtk_tree_view_column_constructor (GType                  type,
+                                  guint                  n_construct_properties,
+                                  GObjectConstructParam *construct_properties)
 {
   GtkTreeViewColumn        *tree_column;
-  GtkTreeViewColumnPrivate *priv;
   GObject                  *object;
 
   object = G_OBJECT_CLASS (gtk_tree_view_column_parent_class)->constructor
     (type, n_construct_properties, construct_properties);
 
   tree_column = (GtkTreeViewColumn *) object;
-  priv        = tree_column->priv;
 
-  if (!priv->cell_area)
-    {
-      priv->cell_area = gtk_cell_area_box_new ();
-      g_object_ref_sink (priv->cell_area);
-    }
-
-  gtk_cell_area_set_style_detail (priv->cell_area, "treeview");
-
-  priv->add_editable_signal =
-    g_signal_connect (priv->cell_area, "add-editable",
-		      G_CALLBACK (gtk_tree_view_column_add_editable_callback),
-		      tree_column);
-  priv->remove_editable_signal =
-    g_signal_connect (priv->cell_area, "remove-editable",
-		      G_CALLBACK (gtk_tree_view_column_remove_editable_callback),
-		      tree_column);
-
-  priv->cell_area_context = gtk_cell_area_create_context (priv->cell_area);
-
-  priv->context_changed_signal =
-    g_signal_connect (priv->cell_area_context, "notify",
-		      G_CALLBACK (gtk_tree_view_column_context_changed), tree_column);
+  gtk_tree_view_column_ensure_cell_area (tree_column, NULL);
 
   return object;
 }
@@ -539,7 +519,7 @@ gtk_tree_view_column_dispose (GObject *object)
 				   priv->add_editable_signal);
       g_signal_handler_disconnect (priv->cell_area,
 				   priv->remove_editable_signal);
-      
+
       g_object_unref (priv->cell_area);
       priv->cell_area = NULL;
       priv->add_editable_signal = 0;
@@ -664,9 +644,18 @@ gtk_tree_view_column_set_property (GObject         *object,
       area = g_value_get_object (value);
 
       if (area)
-	tree_column->priv->cell_area = g_object_ref_sink (area);
+        {
+          if (tree_column->priv->cell_area != NULL)
+            {
+              g_warning ("cell-area has already been set, ignoring construct property");
+              g_object_ref_sink (area);
+              g_object_unref (area);
+            }
+          else
+            gtk_tree_view_column_ensure_cell_area (tree_column, area);
+        }
       break;
-      
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -782,12 +771,51 @@ gtk_tree_view_column_get_property (GObject         *object,
 
 /* Implementation of GtkCellLayout interface
  */
+
+static void
+gtk_tree_view_column_ensure_cell_area (GtkTreeViewColumn *column,
+                                       GtkCellArea       *cell_area)
+{
+  GtkTreeViewColumnPrivate *priv = column->priv;
+
+  if (priv->cell_area)
+    return;
+
+  if (cell_area)
+    priv->cell_area = cell_area;
+  else
+    priv->cell_area = gtk_cell_area_box_new ();
+
+  g_object_ref_sink (priv->cell_area);
+
+ priv->cell_area_context = gtk_cell_area_create_context (priv->cell_area);
+
+  priv->add_editable_signal =
+    g_signal_connect (priv->cell_area, "add-editable",
+                      G_CALLBACK (gtk_tree_view_column_add_editable_callback),
+                      column);
+  priv->remove_editable_signal =
+    g_signal_connect (priv->cell_area, "remove-editable",
+                      G_CALLBACK (gtk_tree_view_column_remove_editable_callback),
+                      column);
+
+  priv->cell_area_context = gtk_cell_area_create_context (priv->cell_area);
+
+  priv->context_changed_signal =
+    g_signal_connect (priv->cell_area_context, "notify",
+                      G_CALLBACK (gtk_tree_view_column_context_changed),
+                      column);
+}
+
 static GtkCellArea *
 gtk_tree_view_column_cell_layout_get_area (GtkCellLayout   *cell_layout)
 {
   GtkTreeViewColumn        *column = GTK_TREE_VIEW_COLUMN (cell_layout);
   GtkTreeViewColumnPrivate *priv   = column->priv;
 
+  if (G_UNLIKELY (!priv->cell_area))
+    gtk_tree_view_column_ensure_cell_area (column, NULL);
+
   return priv->cell_area;
 }
 
@@ -1515,6 +1543,45 @@ _gtk_tree_view_column_get_cell_at_pos (GtkTreeViewColumn *column,
   return match;
 }
 
+gboolean
+_gtk_tree_view_column_is_blank_at_pos (GtkTreeViewColumn *column,
+                                       GdkRectangle      *cell_area,
+                                       GdkRectangle      *background_area,
+                                       gint               x,
+                                       gint               y)
+{
+  GtkCellRenderer *match;
+  GdkRectangle cell_alloc, aligned_area, inner_area;
+  GtkTreeViewColumnPrivate *priv = column->priv;
+
+  match = _gtk_tree_view_column_get_cell_at_pos (column,
+                                                 cell_area,
+                                                 background_area,
+                                                 x, y);
+  if (!match)
+    return FALSE;
+
+  gtk_cell_area_get_cell_allocation (priv->cell_area,
+                                     priv->cell_area_context,
+                                     priv->tree_view,
+                                     match,
+                                     cell_area,
+                                     &cell_alloc);
+
+  gtk_cell_area_inner_cell_area (priv->cell_area, priv->tree_view,
+                                 &cell_alloc, &inner_area);
+  gtk_cell_renderer_get_aligned_area (match, priv->tree_view, 0,
+                                      &inner_area, &aligned_area);
+
+  if (x < aligned_area.x ||
+      x > aligned_area.x + aligned_area.width ||
+      y < aligned_area.y ||
+      y > aligned_area.y + aligned_area.height)
+    return TRUE;
+
+  return FALSE;
+}
+
 /* Public Functions */
 
 
@@ -1731,7 +1798,7 @@ gtk_tree_view_column_set_attributes (GtkTreeViewColumn *tree_column,
  * gtk_tree_view_column_set_cell_data_func:
  * @tree_column: A #GtkTreeViewColumn
  * @cell_renderer: A #GtkCellRenderer
- * @func: The #GtkTreeViewColumnFunc to use. 
+ * @func: (allow-none): The #GtkTreeViewColumnFunc to use. 
  * @func_data: The user data for @func.
  * @destroy: The destroy notification for @func_data
  * 
@@ -1844,6 +1911,8 @@ gtk_tree_view_column_set_visible (GtkTreeViewColumn *tree_column,
   if (priv->visible)
     _gtk_tree_view_column_cell_set_dirty (tree_column, TRUE);
 
+  _gtk_tree_view_reset_header_styles (GTK_TREE_VIEW (priv->tree_view));
+
   gtk_tree_view_column_update_button (tree_column);
   g_object_notify (G_OBJECT (tree_column), "visible");
 }
@@ -2753,10 +2822,10 @@ gtk_tree_view_column_cell_set_cell_data (GtkTreeViewColumn *tree_column,
  * gtk_tree_view_column_cell_get_size:
  * @tree_column: A #GtkTreeViewColumn.
  * @cell_area: (allow-none): The area a cell in the column will be allocated, or %NULL
- * @x_offset: (allow-none): location to return x offset of a cell relative to @cell_area, or %NULL
- * @y_offset: (allow-none): location to return y offset of a cell relative to @cell_area, or %NULL
- * @width: (allow-none): location to return width needed to render a cell, or %NULL
- * @height: (allow-none): location to return height needed to render a cell, or %NULL
+ * @x_offset: (out) (allow-none): location to return x offset of a cell relative to @cell_area, or %NULL
+ * @y_offset: (out) (allow-none): location to return y offset of a cell relative to @cell_area, or %NULL
+ * @width: (out) (allow-none): location to return width needed to render a cell, or %NULL
+ * @height: (out) (allow-none): location to return height needed to render a cell, or %NULL
  * 
  * Obtains the width and height needed to render the column.  This is used
  * primarily by the #GtkTreeView.
@@ -2955,9 +3024,10 @@ _gtk_tree_view_column_cell_get_dirty (GtkTreeViewColumn  *tree_column)
  * gtk_tree_view_column_cell_get_position:
  * @tree_column: a #GtkTreeViewColumn
  * @cell_renderer: a #GtkCellRenderer
- * @x_offset: return location for the horizontal position of @cell within
- *            @tree_column, may be %NULL
- * @width: return location for the width of @cell, may be %NULL
+ * @x_offset: (out) (allow-none): return location for the horizontal
+ *            position of @cell within @tree_column, may be %NULL
+ * @width: (out) (allow-none): return location for the width of @cell,
+ *         may be %NULL
  *
  * Obtains the horizontal position and size of a cell in a column. If the
  * cell is not found in the column, @start_pos and @width are not changed and
@@ -2972,7 +3042,7 @@ gtk_tree_view_column_cell_get_position (GtkTreeViewColumn *tree_column,
 					gint              *width)
 {
   GtkTreeViewColumnPrivate *priv;
-  GdkRectangle zero_cell_area = { 0, };
+  GdkRectangle cell_area;
   GdkRectangle allocation;
 
   g_return_val_if_fail (GTK_IS_TREE_VIEW_COLUMN (tree_column), FALSE);
@@ -2980,16 +3050,24 @@ gtk_tree_view_column_cell_get_position (GtkTreeViewColumn *tree_column,
 
   priv = tree_column->priv;
 
-  /* FIXME: Could use a boolean return value for invalid cells */
+  gtk_tree_view_get_background_area (GTK_TREE_VIEW (priv->tree_view),
+                                     NULL, tree_column, &cell_area);
+
   gtk_cell_area_get_cell_allocation (priv->cell_area,
                                      priv->cell_area_context,
                                      priv->tree_view,
                                      cell_renderer,
-                                     &zero_cell_area,
+                                     &cell_area,
                                      &allocation);
 
   if (x_offset)
-    *x_offset = allocation.x;
+    {
+      GdkRectangle button_allocation;
+
+      /* Retrieve column offset */
+      gtk_widget_get_allocation (priv->button, &button_allocation);
+      *x_offset = allocation.x - button_allocation.x;
+    }
   if (width)
     *width = allocation.width;
 
