@@ -21,9 +21,7 @@
 
 #include <string.h>
 
-#undef GTK_DISABLE_DEPRECATED
-
-#include <gtk/gtk.h>
+#include <gtk/gtkx.h>
 
 #include "gailwindow.h"
 #include "gailtoplevel.h"
@@ -225,12 +223,12 @@ gail_window_real_initialize (AtkObject *obj,
       const gchar *name;
 
       name = gtk_widget_get_name (widget);
-      if (name && (!strcmp (name, "gtk-tooltip") ||
-                   !strcmp (name, "gtk-tooltips")))
+
+      if (!g_strcmp0 (name, "gtk-tooltip"))
         obj->role = ATK_ROLE_TOOL_TIP;
       else if (GTK_IS_PLUG (widget))
         obj->role = ATK_ROLE_PANEL;
-      else if (GTK_WINDOW (widget)->type == GTK_WINDOW_POPUP)
+      else if (gtk_window_get_window_type (GTK_WINDOW (widget)) == GTK_WINDOW_POPUP)
         obj->role = ATK_ROLE_WINDOW;
       else
         obj->role = ATK_ROLE_FRAME;
@@ -274,7 +272,7 @@ gail_window_get_name (AtkObject *accessible)
       /*
        * Get the window title if it exists
        */
-      GtkWidget* widget = GTK_ACCESSIBLE (accessible)->widget; 
+      GtkWidget* widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (accessible));
 
       if (widget == NULL)
         /*
@@ -340,7 +338,7 @@ gail_window_get_parent (AtkObject *accessible)
 static gint
 gail_window_get_index_in_parent (AtkObject *accessible)
 {
-  GtkWidget* widget = GTK_ACCESSIBLE (accessible)->widget; 
+  GtkWidget* widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (accessible));
   AtkObject* atk_obj = atk_get_root ();
   gint index = -1;
 
@@ -401,7 +399,7 @@ gail_window_ref_relation_set (AtkObject *obj)
 
   gail_return_val_if_fail (GAIL_IS_WIDGET (obj), NULL);
 
-  widget = GTK_ACCESSIBLE (obj)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (obj));
   if (widget == NULL)
     /*
      * State is defunct
@@ -418,7 +416,7 @@ gail_window_ref_relation_set (AtkObject *obj)
         {
           atk_relation_set_remove (relation_set, relation);
         }
-      if (gtk_widget_get_visible(widget) && gtk_tooltips_get_info_from_tip_window (GTK_WINDOW (widget), NULL, &current_widget))
+      if (gtk_widget_get_visible(widget) && FALSE /* FIXME gtk_tooltips_get_info_from_tip_window (GTK_WINDOW (widget), NULL, &current_widget) */)
         {
           array [0] = gtk_widget_get_accessible (current_widget);
 
@@ -436,22 +434,24 @@ gail_window_ref_state_set (AtkObject *accessible)
   AtkStateSet *state_set;
   GtkWidget *widget;
   GtkWindow *window;
+  GdkWindow *gdk_window;
   GdkWindowState state;
 
   state_set = ATK_OBJECT_CLASS (gail_window_parent_class)->ref_state_set (accessible);
-  widget = GTK_ACCESSIBLE (accessible)->widget;
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (accessible));
  
   if (widget == NULL)
     return state_set;
 
   window = GTK_WINDOW (widget);
 
-  if (window->has_focus)
+  if (gtk_window_has_toplevel_focus (window) && gtk_window_is_active (window))
     atk_state_set_add_state (state_set, ATK_STATE_ACTIVE);
 
-  if (widget->window)
+  gdk_window = gtk_widget_get_window (widget);
+  if (window)
     {
-      state = gdk_window_get_state (widget->window);
+      state = gdk_window_get_state (gdk_window);
       if (state & GDK_WINDOW_STATE_ICONIFIED)
         atk_state_set_add_state (state_set, ATK_STATE_ICONIFIED);
     } 
@@ -472,7 +472,7 @@ idle_notify_name_change (gpointer data)
 
   window = GAIL_WINDOW (data);
   window->name_change_handler = 0;
-  if (GTK_ACCESSIBLE (window)->widget == NULL)
+  if (gtk_accessible_get_widget (GTK_ACCESSIBLE (window)) == NULL)
     return FALSE;
 
   obj = ATK_OBJECT (window);
@@ -551,7 +551,7 @@ gail_window_get_extents (AtkComponent  *component,
                          gint          *height,
                          AtkCoordType  coord_type)
 {
-  GtkWidget *widget = GTK_ACCESSIBLE (component)->widget; 
+  GtkWidget *widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (component)); 
   GdkRectangle rect;
   gint x_toplevel, y_toplevel;
 
@@ -572,7 +572,8 @@ gail_window_get_extents (AtkComponent  *component,
       return;
     }
 
-  gdk_window_get_frame_extents (widget->window, &rect);
+  gdk_window_get_frame_extents (gtk_widget_get_window (widget),
+                                &rect);
 
   *width = rect.width;
   *height = rect.height;
@@ -586,7 +587,8 @@ gail_window_get_extents (AtkComponent  *component,
   *y = rect.y;
   if (coord_type == ATK_XY_WINDOW)
     {
-      gdk_window_get_origin (widget->window, &x_toplevel, &y_toplevel);
+      gdk_window_get_origin (gtk_widget_get_window (widget),
+                             &x_toplevel, &y_toplevel);
       *x -= x_toplevel;
       *y -= y_toplevel;
     }
@@ -597,7 +599,7 @@ gail_window_get_size (AtkComponent *component,
                       gint         *width,
                       gint         *height)
 {
-  GtkWidget *widget = GTK_ACCESSIBLE (component)->widget; 
+  GtkWidget *widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (component));
   GdkRectangle rect;
 
   if (widget == NULL)
@@ -616,7 +618,7 @@ gail_window_get_size (AtkComponent *component,
       parent_iface->get_size (component, width, height);
       return;
     }
-  gdk_window_get_frame_extents (widget->window, &rect);
+  gdk_window_get_frame_extents (gtk_widget_get_window (widget), &rect);
 
   *width = rect.width;
   *height = rect.height;
@@ -662,10 +664,10 @@ get_window_desktop (Window window)
 
   if (_net_wm_desktop == None)
     _net_wm_desktop =
-		XInternAtom (gdk_display, "_NET_WM_DESKTOP", False);
+		XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), "_NET_WM_DESKTOP", False);
 
   gdk_error_trap_push ();
-  result = XGetWindowProperty (gdk_display, window, _net_wm_desktop,
+  result = XGetWindowProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), window, _net_wm_desktop,
                                0, G_MAXLONG,
                                False, XA_CARDINAL,
                                &ret_type, &format, &nitems,
@@ -716,12 +718,12 @@ get_stacked_windows (GailScreenInfo *info)
 
   if (_net_client_list_stacking == None)
     _net_client_list_stacking =
-		XInternAtom (gdk_display, "_NET_CLIENT_LIST_STACKING", False);
+		XInternAtom (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), "_NET_CLIENT_LIST_STACKING", False);
 
   gdk_error_trap_push ();
   ret_type = None;
-  result = XGetWindowProperty (gdk_display,
-                               GDK_WINDOW_XWINDOW (info->root_window),
+  result = XGetWindowProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+                               GDK_WINDOW_XID (info->root_window),
                                _net_client_list_stacking,
                                0, G_MAXLONG,
                                False, XA_WINDOW, &ret_type, &format, &nitems,
@@ -825,8 +827,7 @@ filter_func (GdkXEvent *gdkxevent,
 
           if (window)
             {
-              screen_n = gdk_screen_get_number (
-                  gdk_drawable_get_screen (GDK_DRAWABLE (window)));
+              screen_n = gdk_screen_get_number (gdk_window_get_screen (window));
 
               gail_screens [screen_n].update_stacked_windows = TRUE;
               if (!gail_screens [screen_n].update_handler)
@@ -917,12 +918,12 @@ init_gail_screen (GdkScreen *screen,
 
   get_stacked_windows (&gail_screens [screen_n]);
 
-  XGetWindowAttributes (gdk_display,
-			GDK_WINDOW_XWINDOW (gail_screens [screen_n].root_window),
+  XGetWindowAttributes (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+			GDK_WINDOW_XID (gail_screens [screen_n].root_window),
 			&attrs); 
 
-  XSelectInput (gdk_display,
-		GDK_WINDOW_XWINDOW (gail_screens [screen_n].root_window),
+  XSelectInput (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+		GDK_WINDOW_XID (gail_screens [screen_n].root_window),
 		attrs.your_event_mask | PropertyChangeMask);
            
   gail_screens [screen_n].screen_initialized = TRUE;
@@ -963,8 +964,7 @@ get_window_zorder (GdkWindow *window)
 
   gail_return_val_if_fail (GDK_IS_WINDOW (window), -1);
 
-  info = get_screen_info (
-		gdk_drawable_get_screen (GDK_DRAWABLE (window)));
+  info = get_screen_info (gdk_window_get_screen (window));
 
   gail_return_val_if_fail (info->stacked_windows != NULL, -1);
 
@@ -1002,7 +1002,7 @@ get_window_zorder (GdkWindow *window)
 static gint
 gail_window_get_mdi_zorder (AtkComponent *component)
 {
-  GtkWidget *widget = GTK_ACCESSIBLE (component)->widget;
+  GtkWidget *widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (component));
 
   if (widget == NULL)
     /*
@@ -1012,7 +1012,7 @@ gail_window_get_mdi_zorder (AtkComponent *component)
 
   gail_return_val_if_fail (GTK_IS_WINDOW (widget), -1);
 
-  return get_window_zorder (widget->window);
+  return get_window_zorder (gtk_widget_get_window (widget));
 }
 
 #elif defined (GDK_WINDOWING_WIN32)
@@ -1020,7 +1020,7 @@ gail_window_get_mdi_zorder (AtkComponent *component)
 static gint
 gail_window_get_mdi_zorder (AtkComponent *component)
 {
-  GtkWidget *widget = GTK_ACCESSIBLE (component)->widget;
+  GtkWidget *widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (component));
 
   if (widget == NULL)
     /*
@@ -1038,7 +1038,7 @@ gail_window_get_mdi_zorder (AtkComponent *component)
 static gint
 gail_window_get_mdi_zorder (AtkComponent *component)
 {
-  GtkWidget *widget = GTK_ACCESSIBLE (component)->widget;
+  GtkWidget *widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (component));
 
   if (widget == NULL)
     /*
